@@ -550,10 +550,11 @@ def CLF(X, y, verbose = False, show = False, save_fig = ''):
     for yv in set(y):
         grp_samples.append((y == yv).sum())   
         
-    # min(grp_samples) is the minimum sample size among all categories. CV requires to be not greater than this value.
+    # min(grp_samples) is the minimum sample size among all categories. 
+    # CV requires to be not greater than this value.
 
     try:
-        clf = LogisticRegressionCV(cv = min(10, min(grp_samples)), max_iter = 1000).fit(X, y) # ridge(L2) regularization
+        clf = LogisticRegressionCV(cv = min(3, min(grp_samples)), max_iter = 1000).fit(X, y) # ridge(L2) regularization
     except:
         print('Exception in LogisticRegressionCV().')
         return None,None,None
@@ -617,7 +618,7 @@ def CLF(X, y, verbose = False, show = False, save_fig = ''):
             
     y_pred = clf.predict(X)
     clf_metrics.append( globals()["accuracy_score"](y, y_pred) )
-    clf_metrics.append( globals()["cohen_kappa_score"](y, y_pred) ) # use ground truth and prediction as 1st and 2nd raters
+    clf_metrics.append( globals()["cohen_kappa_score"](y, y_pred) ) # use ground truth and prediction as 1st and 2nd raters. 0 - no agreement, 1 - perfect agreement.
     clf_metrics.append( globals()["f1_score"](y, y_pred) )
     clf_metrics.append( globals()["jaccard_score"](y, y_pred) )
     clf_metrics.append( globals()["precision_score"](y, y_pred) )
@@ -649,7 +650,7 @@ def CLF(X, y, verbose = False, show = False, save_fig = ''):
     clf_metrics.append( mkld )
     
     clf_metrics.append( globals()["average_precision_score"](y, y_prob) )
-    clf_metrics.append( globals()["brier_score_loss"](y, y_prob) )
+    clf_metrics.append( globals()["brier_score_loss"](y, y_prob) ) # The Brier score measures the mean squared difference between the predicted probability and the actual outcome.
     clf_metrics.append( globals()["roc_auc_score"](y, y_prob) )
     
     precisions, recalls, _ = precision_recall_curve(y, y_prob, pos_label = max(y)) # set pos_label for cases when y is not {0,1} or {-1,1}
@@ -1243,6 +1244,10 @@ def MWW(X,y, verbose = False, show = False, max_plot_num = 5):
 
     return ps, Us, IMG
 
+def es_max(X,y):
+    d, _ = cohen_d(X,y)
+    return d.max()
+
 def cohen_d(X, y, show = False, save_fig = ''):
     '''
     Cohen’s d is a type of effect size between two means. Cohen’s d values are also known as the standardised mean difference (SMD).
@@ -1352,6 +1357,7 @@ def correlate(X,y, verbose = False, show = False):
     dic['correlation.r2'] = np.power(rs,2) # R2, the R-squared effect size
     dic['correlation.r.p'] = prs
     dic['correlation.r.max'] = np.abs(rs).max() # abs max
+    dic['correlation.r2.max'] = np.power(rs,2).max() # abs max
     dic['correlation.r.p.min'] = np.min(prs)
     
     dic['correlation.rho'] = rhos
@@ -1678,104 +1684,84 @@ def metrics_keys():
 
 
 # Defines how each metric's polarity, i.e., how to compare each metric.  
-#   np.argmax - the metric and classifiability are positively correlated
-#   np.argmin - the metric and classifiability are negatively correlated
+#   np.nanargmax - the metric and classifiability are positively correlated
+#   np.nanargmin - the metric and classifiability are negatively correlated
 metric_polarity_dict={
-    'classification.ACC': np.argmax,
-    'classification.Kappa': np.argmax,
-    'classification.F1_Score': np.argmax ,
-    'classification.Jaccard': np.argmin,
-    'classification.Precision': np.argmax,
-    'classification.Recall': np.argmax,
-    'classification.McNemar': np.argmin,
-    'classification.McNemar.CHI2':np.argmax,
-    'classification.CochranQ': np.argmin,
-    'classification.CochranQ.T':np.argmax,
-    'classification.CrossEntropy':np.argmin,
-    'classification.Mean_KLD': np.argmin,
-    'classification.AP': np.argmax,
-    'classification.Brier':np.argmin ,
-    'classification.ROC_AUC':np.argmax,
-    'classification.PR_AUC':np.argmin,
-    'classification.BER':np.argmin,
-    'classification.SVM.Margin':np.argmax,
-    'correlation.IG':np.argmax,
-    'correlation.IG.max':np.argmax,
-    'correlation.r':np.argmax,
-    'correlation.r2':np.argmax,
-    'correlation.r.p':np.argmin,
-    'correlation.r.max':np.argmax,
-    'correlation.r.p.min':np.argmin,
-    'correlation.rho':np.argmax,
-    'correlation.rho.p':np.argmin,
-    'correlation.rho.max':np.argmax,
-    'correlation.rho.p.min':np.argmin,
-    'correlation.tau':np.argmin,
-    'correlation.tau.p':np.argmin,
-    'correlation.tau.max':np.argmax,
-    'correlation.tau.p.min':np.argmin,
-    'test.ES':np.argmax,
-    'test.ES.max':np.argmax,
-    'test.student':np.argmin,
-    'test.student.min':np.argmin,
-    'test.student.min.log10':np.argmin,
-    'test.student.T':np.argmax,
-    'test.student.T.max':np.argmin,
-    'test.ANOVA':np.argmin,
-    'test.ANOVA.min':np.argmin,
-    'test.ANOVA.min.log10':np.argmin,
-    'test.ANOVA.F':np.argmax,
-    'test.ANOVA.F.max':np.argmax,
-    'test.MANOVA':np.argmin,
-    'test.MANOVA.log10':np.argmin,
-    'test.MANOVA.F':np.argmax,
-    'test.MWW':np.argmin,
-    'test.MWW.min':np.argmin,
-    'test.MWW.min.log10':np.argmin,
-    'test.MWW.U':np.argmin,
-    'test.MWW.U.min':np.argmin,
-    'test.KS':np.argmin,
-    'test.KS.min':np.argmin,
-    'test.KS.min.log10':np.argmin,
-    'test.KS.D':np.argmax,
-    'test.KS.D.max':np.argmax,
-    'test.CHISQ':np.argmin,
-    'test.CHISQ.min':np.argmin,
-    'test.CHISQ.min.log10':np.argmin,
-    'test.CHISQ.CHI2':np.argmax,
-    'test.CHISQ.CHI2.max':np.argmax,
-    'test.KW':np.argmin,
-    'test.KW.min':np.argmin,
-    'test.KW.min.log10':np.argmin,
-    'test.KW.H':np.argmax,
-    'test.KW.H.max':np.argmax,
-    'test.Median':np.argmin,
-    'test.Median.min':np.argmin,
-    'test.Median.min.log10':np.argmin,
-    'test.Median.CHI2':np.argmax,
-    'test.Median.CHI2.max':np.argmax,
-    'overlapping.F1.mean':np.argmax,
-    'overlapping.F1.sd':np.argmax,
-    'overlapping.F1v.mean':np.argmax,
-    'overlapping.F1v.sd':np.argmin,
-    'overlapping.F2.mean':np.argmin,
-    'overlapping.F2.sd':np.argmin,
-    'overlapping.F3.mean':np.argmax,
-    'overlapping.F3.sd':np.argmin,
-    'overlapping.F4.mean':np.argmax,
-    'overlapping.F4.sd':np.argmin,
-    'neighborhood.N1':np.argmin,
-    'neighborhood.N2.mean':np.argmin,
-    'neighborhood.N2.sd':np.argmin,
-    'neighborhood.N3.mean':np.argmin,
-    'neighborhood.N3.sd':np.argmax,
-    'neighborhood.N4.mean':np.argmin,
-    'neighborhood.N4.sd':np.argmin,
-    'neighborhood.T1.mean':np.argmax,
-    'neighborhood.T1.sd':np.argmin,
-    'neighborhood.LSC':np.argmax,
-    'linearity.L1.mean':np.argmin,
-    'linearity.L1.sd':np.argmin
+    'classification.ACC': np.nanargmax,
+    'classification.Kappa': np.nanargmax,
+    'classification.F1_Score': np.nanargmax ,
+    'classification.Jaccard': np.nanargmin, # IOU between pred and truch
+    'classification.Precision': np.nanargmax,
+    'classification.Recall': np.nanargmax,
+    'classification.McNemar': np.nanargmax, # we want see no diff between pred and truth
+    'classification.McNemar.CHI2':np.nanargmax,
+    'classification.CochranQ': np.nanargmax, # we want see no diff between pred and truth
+    'classification.CochranQ.T':np.nanargmin, 
+    'classification.CrossEntropy':np.nanargmin,
+    'classification.Mean_KLD': np.nanargmin,
+    'classification.AP': np.nanargmax,
+    'classification.Brier':np.nanargmin ,
+    'classification.ROC_AUC':np.nanargmax,
+    'classification.PR_AUC':np.nanargmax,
+    'classification.BER':np.nanargmin,
+    'classification.SVM.Margin':np.nanargmax,
+    'correlation.IG.max':np.nanargmax,
+    'correlation.r.p':np.nanargmin,
+    'correlation.r.max':np.nanargmax,
+    'correlation.r2.max':np.nanargmax,
+    'correlation.r.p.min':np.nanargmin,
+    'correlation.rho.max':np.nanargmax,
+    'correlation.rho.p.min':np.nanargmin,
+    'correlation.tau.max':np.nanargmax,
+    'correlation.tau.p.min':np.nanargmin,
+    'test.ES.max':np.nanargmax,
+    'test.student':np.nanargmin,
+    'test.student.min':np.nanargmin,
+    'test.student.min.log10':np.nanargmin,
+    'test.student.T.max':np.nanargmax,
+    'test.ANOVA.min':np.nanargmin,
+    'test.ANOVA.min.log10':np.nanargmin,
+    'test.ANOVA.F.max':np.nanargmax,
+    'test.MANOVA':np.nanargmin,
+    'test.MANOVA.log10':np.nanargmin,
+    'test.MANOVA.F':np.nanargmax,
+    'test.MWW.min':np.nanargmin,
+    'test.MWW.min.log10':np.nanargmin,
+    'test.MWW.U.min':np.nanargmin,
+    'test.KS.min':np.nanargmin,
+    'test.KS.min.log10':np.nanargmin,
+    'test.KS.D.max':np.nanargmax,
+    'test.CHISQ.min':np.nanargmin,
+    'test.CHISQ.min.log10':np.nanargmin,
+    'test.CHISQ.CHI2.max':np.nanargmax,
+    'test.KW.min':np.nanargmin,
+    'test.KW.min.log10':np.nanargmin,
+    'test.KW.H.max':np.nanargmax,
+    'test.Median.min':np.nanargmin,
+    'test.Median.min.log10':np.nanargmin,
+    'test.Median.CHI2.max':np.nanargmax,
+    'overlapping.F1.mean':np.nanargmax,
+    'overlapping.F1.sd':np.nanargmax,
+    'overlapping.F1v.mean':np.nanargmax,
+    # 'overlapping.F1v.sd':np.nanargmin,
+    'overlapping.F2.mean':np.nanargmin,
+    # 'overlapping.F2.sd':np.nanargmin,
+    'overlapping.F3.mean':np.nanargmax,
+    # 'overlapping.F3.sd':np.nanargmin,
+    'overlapping.F4.mean':np.nanargmax,
+    # 'overlapping.F4.sd':np.nanargmin,
+    'neighborhood.N1':np.nanargmin,
+    'neighborhood.N2.mean':np.nanargmin,
+    # 'neighborhood.N2.sd':np.nanargmin,
+    'neighborhood.N3.mean':np.nanargmin,
+    # 'neighborhood.N3.sd':np.nanargmax,
+    'neighborhood.N4.mean':np.nanargmin,
+    # 'neighborhood.N4.sd':np.nanargmin,
+    'neighborhood.T1.mean':np.nanargmax,
+    # 'neighborhood.T1.sd':np.nanargmin,
+    'neighborhood.LSC':np.nanargmax,
+    'linearity.L1.mean':np.nanargmin,
+    # 'linearity.L1.sd':np.nanargmin
  }
 
 def get_json(X,y):    
